/*******************************************************************************
*  GLCD - グラフィックＬＣＤモジュールに表示を行う                             *
*                                                                              *
*    GLCD_Init()     - LCDモジュールに初期化情報を発行する処理                 *
*    GLCD_gcls()     - LCD画面をクリアする処理                                 *
*    GLCD_FillRange()- LCD画面の指定された場所を反転･消去･塗りの描画を行う処理 *
*    GLCD_PutImage() - LCD画面にイメージデータ(128x64)を描画させる処理         *
*    GLCD_PutsK()    - LCD画面の指定された位置に漢字の文字列を描画させる処理   *
*    GLCD_PutcK()    - LCD画面の指定された位置に漢字１文字描画させる処理       *
*    GLCD_Puts()     - LCD画面の指定された位置に文字列(8x5)を描画させる処理    *
*    GLCD_Putc()     - LCD画面の指定された位置に１文字(8x5)描画させる処理      *
*    GLCD_Box()      - LCD画面の指定された位置に四角形を描画させる処理         *
*    GLCD_Line()     - LCD画面の指定された位置に直線を描画させる処理           *
*    GLCD_PutPixel() - LCD画面の指定された位置にピクセル(ﾄﾞｯﾄ)を描画させる処理 *
*    GLCD_PutPattern()-LCD画面の指定された位置にドットパターンを描画させる処理 *
*                                                                              *
* ============================================================================ *
*  VERSION DATE        BY                    CHANGE/COMMENT                    *
* ---------------------------------------------------------------------------- *
*  01.00   2011-09-05  きむ茶工房(きむしげ)  Create                            *
*  01.10   2011-09-15  きむ茶工房(きむしげ)  GLCD_PutPattern()を追加           *
*  01.11   2011-11-02  きむ茶工房(きむしげ)  GLCD_Box()の縦線描画処理を一部修正*
*******************************************************************************/
#include "font.h"			// 5x8のキャラクター文字列データ用ファイル
#include "KanjiFont.h"		// 12x12の漢字フォントデータ用ファイル
#include "imagedata.h"		// 128x64のイメージデータファイル
#include "PatternData.h"      // ドットパターンデータファイル


int RS_pin ;                  // RS(register select)の接続ピン番号を保存
int RW_pin ;                  // RW(read/write)の接続ピン番号を保存
int E_pin ;                   // E (enable signal)の接続ピン番号を保存
int CS1_pin ;                 // CS1(selection IC1)の接続ピン番号を保存
int CS2_pin ;                 // CS2(selection IC2)の接続ピン番号を保存
int DATA_pins[8] ;            // DB0-7(data bit)の接続ピン番号を保存
int SetPg ;
int SetCol ;


int kdt[10]={12,13,14,15,16,17,18,19,20,0} ; // "漢字のフォント表示"の文字列データ

/*******************************************************************************
*  電源起動時とリセットの時だけのみ処理される関数(初期化と設定処理)            *
*******************************************************************************/
void setup() {
     int ans ;
     
     // グラフィックLCDモジュールと接続するピンの初期化を行う
     // LCDモジュールとArduinoの接続ピンを変更した場合はこの行を変えます。
     Pins_Init(0,4,1,5,2,3,6,7,8,9,10,11,12,13) ;
     // グラフィックLCDモジュールの初期化を行う
     GLCD_Init() ;
     // LCD画面のクリアを行う
     GLCD_gcls() ;

     GLCD_Box(0,0,127,63) ;
     GLCD_PutsK(6,2,kdt) ;
     GLCD_Line(0,15,127,15) ;
}
/*******************************************************************************
*  繰り返し実行される処理の関数(メインの処理)                                  *
*******************************************************************************/
void loop() {
     unsigned long t ;
     char buf[12] ;
  
     // プログラム開始からの時刻を読み出して表示させる
     t = millis() ;
     sprintf(buf,"%010ld",t) ;
     GLCD_Puts(30,38,buf) ;
     // １秒後に繰り返す
     delay(1000) ;
}
/*******************************************************************************
*  Pins_Init(rst,rs,rw,enable,cs1,cs2,d0,d1,d2,d3,d4,d5,d6,d7)                 *
*    グラフィックLCDモジュールと接続するピンの初期化処理                       *
*                                                                              *
*    rst    : RST(reset signal)信号と接続しているピン番号を指定                *
*    rs     : RS(register select)信号と接続しているピン番号を指定              *
*    rw     : RW(read/write)信号と接続しているピン番号を指定                   *
*    enable : E (enable signal)信号と接続しているピン番号を指定                *
*    cs1    : CS1(selection IC1)信号と接続しているピン番号を指定               *
*    cs2    : CS2(selection IC2)信号と接続しているピン番号を指定               *
*    d0-d7  : DB4-7(data bit)信号と接続しているピン番号を指定                  *
*******************************************************************************/
void Pins_Init(int rst,int rs,int rw,int enable,int cs1,int cs2,
               int d0,int d1,int d2,int d3,int d4,int d5,int d6,int d7)
{
     // データピンの番号をセット
     DATA_pins[0] = d0 ;
     DATA_pins[1] = d1 ;
     DATA_pins[2] = d2 ;
     DATA_pins[3] = d3 ;
     DATA_pins[4] = d4 ;
     DATA_pins[5] = d5 ;
     DATA_pins[6] = d6 ;
     DATA_pins[7] = d7 ;
     // 制御ピンの番号をセット
     RS_pin  = rs ;
     RW_pin  = rw ;
     E_pin   = enable ;
     CS1_pin = cs1 ;
     CS2_pin = cs2 ;
     // 指定の制御ピンをデジタル出力に設定
     pinMode(RS_pin, OUTPUT) ;
     pinMode(RW_pin, OUTPUT) ;
     pinMode(E_pin, OUTPUT) ;
     pinMode(CS1_pin, OUTPUT) ;
     pinMode(CS2_pin, OUTPUT) ;
     // 指定のデータピンをデジタル出力に設定
     for (int i = 0; i < 8; i++) pinMode(DATA_pins[i], OUTPUT) ;
     // 信号線をLOWに設定して置きます
     digitalWrite(RS_pin, LOW);
     digitalWrite(RW_pin, LOW);
     digitalWrite(E_pin, LOW);
     digitalWrite(CS1_pin, LOW);
     digitalWrite(CS2_pin, LOW);
     // グラフィックLCDモジュールのリセットを解除する
     pinMode(rst, OUTPUT) ;
     digitalWrite(rst, HIGH);
}
/*******************************************************************************
*  GLCD_Init()                                                                 *
*    グラフィックLCDモジュールに初期化情報を発行する処理                       *
*******************************************************************************/
void GLCD_Init()
{
     // LCD電源がONしてから30msほどウエイトするみたいなので入れておく
     delay(30) ;
     // 左側ディスプレイのチップ(IC1)を初期化する
     SelectIC(1) ;		     // IC1を選択
     command(0xC0,LOW) ;		// Display start lineのコマンドを発行する
     command(0x3F,LOW) ;		// Display Onのコマンドを発行する
     // 右側ディスプレイのチップ(IC2)を初期化する
     SelectIC(2) ;            // IC2を選択
     command(0xC0,LOW) ;		// Display start lineのコマンドを発行する
     command(0x3F,LOW) ;		// Display Onのコマンドを発行する
}
/*******************************************************************************
*  GLCD_gcls()                                                                 *
*    ディスプレイの画面をクリアする処理（画面全体に０のデータを設定しています）*
*******************************************************************************/
void GLCD_gcls()
{
     int x , y , i ;

     for (i=1 ; i<3 ; i++ ) {
          SelectIC(i) ;			// ICチップの選択
          // ８ページ分のデータ
          for (y=0 ; y<8 ; y++) {
               SetPage(y) ;		// ページの設定
               SetAddress(0) ;	// カラムの設定
               // １ページ分のデータ、ゼロを送る
               for (x=0 ; x<64 ; x++) {
		          WriteData(0) ;	// 次のカラム設定は自動的にLCDが進める
               }
          }
     }
}
/*******************************************************************************
*  GLCD_FillRange(Xp0,Yp0,Xp1,Yp1,mode)                                        *
*    LCD画面の指定された位置の四角形エリアを指定のモード(mode)で描画させる処理 *
*                                                                              *
*    Xp0 : 反転する四角形エリアの左上角点、横アドレス(0-127)を指定する         *
*    Yp0 : 反転する四角形エリアの左上角点、縦アドレス(0-63)を指定する          *
*    Xp1 : 反転する四角形エリアの右下角点、横アドレス(0-127)を指定する         *
*    Yp1 : 反転する四角形エリアの右下角点、縦アドレス(0-63)を指定する          *
*    mode: 0=エリアを消す  1=エリアを塗りつぶす  2=反転する                    *
*******************************************************************************/
void GLCD_FillRange(int Xp0,int Yp0,int Xp1,int Yp1,int mode)
{
     int x , y , L ;

     for (x=Xp0 ; x <= Xp1 ; x++) {
          for (y=Yp0 ; y <= Yp1 ; y++) {
               // LCDに表示するアドレスの位置をセットする
               SetLocation(x,y) ;
               SetAddress(SetCol) ;
               // ライン(ビット位置)の選択処理
               L = 1 << (y % 8) ;
               if (mode == 0) L = ReadData() & ~L ; // 指定位置のビット(L)をOFFさせる
               if (mode == 1) L = ReadData() | L ;  // 指定位置のビット(L)をONさせる
               if (mode == 2) L = ReadData() ^ L ;  // 指定位置のビット(L)とXORさせる
               // データをLCDに送る
               SetAddress(SetCol) ;
               WriteData(L) ;
          }
     }
}
/*******************************************************************************
*  GLCD_PutPattern(col,pg,p)                                                   *
*    グラフィックLCD画面にドットパターンを描画させる処理                       *
*    (描画位置の縦指定:pgはピクセルでなく、ページ位置です注意)                 *
*                                                                              *
*    col : 描画する位置で画面の横アドレス(0-127)のカラム位置を指定する         *
*    pg  : 描画する位置で画面の縦アドレス(0-7)のページ位置を指定する           *
*    *p  : 描画するドットパターンデータを格納した場所を指定する                *
*******************************************************************************/
void GLCD_PutPattern(int col,int pg, const char * p)
{
     int i , j , xs , ys ; 

     // ドットパターンを描画する
     xs = *p++ ;
     ys = *p++ / 8 ;
     for (j=0; j<ys; j++) {
          for(i=0; i<xs; i++) {
               // 描画位置を設定す
               SetLocation(col+i,(pg+j)*8) ;
               // データを出力する
               WriteData(*p++);
          }
     }
}
/*******************************************************************************
*  GLCD_PutImage(p)                                                            *
*    グラフィックLCD画面にイメージデータ(128x64)を描画させる処理               *
*                                                                              *
*    *p : 描画するイメージデータを格納した場所を指定する                       *
*******************************************************************************/
void GLCD_PutImage(const char * p)
{
	int cs, x;
	int page, colum;

	for(page=0 ; page<8 ; page++) {
		for(colum=0 ; colum<128 ; colum++) {
               // チップの選択とカラムアドレスの処理を行う
			if(colum > 63) {
                    // 右側ディスプレイのチップ(IC2)とする
				x  = colum - 64 ;
				cs = 2;
			} else {
                    // 左側ディスプレイのチップ(IC1)とする
				x  = colum ;
				cs = 1 ;
			}
               // 描画位置の設定を行う
               SelectIC(cs) ;
               SetPage(page) ;
               SetAddress(x) ;
               // データを送る
               WriteData(*p++);
		}
	}
}
/*******************************************************************************
*  GLCD_PutsK(Xp,Yp,s)                                                         *
*    グラフィックLCD画面の指定された位置に漢字の文字列を描画させる処理         *
*                                                                              *
*    Xp : 描画する位置で画面の横アドレス(0-127)を指定する                      *
*    Yp : 描画する位置で画面の縦アドレス(0-63)を指定する                       *
*    *s : 描画する漢字ﾃﾞｰﾀ配列のｲﾝﾃﾞｯｸｽを格納した場所を指定(Null[0x00]まで出力)*
*******************************************************************************/
void GLCD_PutsK(int Xp,int Yp,const int * s)
{
     int x ;

     x = Xp ;
     while(*s) {
          GLCD_PutcK(x,Yp,*s++) ;
          x = x + 13 ;  // 文字と文字の間は１ドットあける
     }
}
/*******************************************************************************
*  GLCD_PutcK(Xp,Yp,Index)                                                     *
*    グラフィックLCD画面の指定された位置に漢字１文字描画させる処理             *
*    （漢字は12x12だが実際は12x16で出力される、下4ﾄﾞｯﾄはスペース[空白]です）   *
*                                                                              *
*    Xp    : 描画する位置で画面の横アドレス(0-127)を指定する                   *
*    Yp    : 描画する位置で画面の縦アドレス(0-63)を指定する                    *
*    Index : 描画する漢字データ配列(KanjiFont[ここ][])のインデックスを指定する *
*            （KanjiFont[5][]なら0-4指定でなく1-5で指定する)                   *
*******************************************************************************/
void GLCD_PutcK(int Xp,int Yp,int Index)
{
     int index ;
     int p , L , f , i ; 

     // ページ内のラインを選択
     L = Yp % 8 ;
     index = Index - 1 ;
     for(p=0; p<12; p++) {
          // 描画位置を設定する
          SetLocation(Xp+p,Yp) ;
          // フォントデータの上側を出力する
          // 描画位置の表示しているデータとフォントデータをXORして出力する
          f = ReadData() ^ (KanjiFont[index][p] << L) ;
          SetAddress(SetCol) ;
          WriteData(f);
          i = p + 12 ;
          if ((L != 0) && (SetPg < 6)) {
               // フォントデータの上側が次のページにまたがった場合の処理
               SetPg++ ;
               SetPage(SetPg) ;
               SetAddress(SetCol) ;
               f = ((unsigned char)KanjiFont[index][p] >> (8-L)) ;
               f = ReadData() ^ (f | (KanjiFont[index][i] << L)) ;
               SetAddress(SetCol) ;
               WriteData(f);
                 f = (unsigned char)KanjiFont[index][i] >> (8-L) ;
          } else f = KanjiFont[index][i] << L ;
          // フォントデータの下側を出力する
          SetPage(SetPg+1) ;
          SetAddress(SetCol) ;
          f = ReadData() ^ f ;
          SetAddress(SetCol) ;
          WriteData(f);
     }
}
/*******************************************************************************
*  GLCD_Puts(Xp,Yp,s)                                                          *
*    グラフィックLCD画面の指定された位置に文字列(8x5)を描画させる処理          *
*                                                                              *
*    Xp : 描画する位置で画面の横アドレス(0-127)を指定する                      *
*    Yp : 描画する位置で画面の縦アドレス(0-63)を指定する                       *
*    *s : 描画する文字列を格納した場所を指定する（Null[0x00]まで出力する）     *
*******************************************************************************/
void GLCD_Puts(int Xp,int Yp,const char * s)
{
     int x ;

     x = Xp ;
     while(*s) {
          GLCD_Putc(x,Yp,*s++) ;
          x = x + 6 ;  // 文字と文字の間は１ドットあける
     }
}
/*******************************************************************************
*  GLCD_Putc(Xp,Yp,c)                                                          *
*    グラフィックLCD画面の指定された位置に１文字(8x5)描画させる処理            *
*                                                                              *
*    Xp : 描画する位置で画面の横アドレス(0-127)を指定する                      *
*    Yp : 描画する位置で画面の縦アドレス(0-63)を指定する                       *
*    c  : 描画する１バイトの文字(キャラクターコード値)を指定する               *
*******************************************************************************/
void GLCD_Putc(int Xp,int Yp,unsigned char c)
{
     int i , L , f ; 

     // ページ内のラインを選択
     L = Yp % 8 ;
     // ８ｘ５のキャラクター文字を描画する
     for(i=0; i<5; i++) {
          // 描画位置を設定する
          SetLocation(Xp+i,Yp) ;
          // フォントデータを出力する
          f = Font[c-0x20][i] << L ;
          WriteData(f);
          // 文字が次ページにまたがっていた場合の処理
          if ((L != 0) && (SetPg < 7)) {
               SetPage(SetPg+1) ;
               SetAddress(SetCol) ;
               // 次のページにまたがった部分の文字データを出力する
               f = (unsigned char)Font[c-0x20][i] >> (8-L) ;
               WriteData(f);
          }
     }
}
/*******************************************************************************
*  GLCD_Box(Xp0,Yp0,Xp1,Yp1)                                                   *
*    グラフィックLCD画面の指定された位置に四角形を描画させる処理               *
*                                                                              *
*    Xp0 : 描画する四角形の左上角点、横アドレス(0-127)を指定する               *
*    Yp0 : 描画する四角形の左上角点、縦アドレス(0-63)を指定する                *
*    Xp1 : 描画する四角形の右下角点、横アドレス(0-127)を指定する               *
*    Yp1 : 描画する四角形の右下角点、縦アドレス(0-63)を指定する                *
*******************************************************************************/
void GLCD_Box(int Xp0,int Yp0,int Xp1,int Yp1)
{
     int i ;

     for (i=Xp0 ; i <= Xp1 ; i++) {
          GLCD_PutPixel(i,Yp0) ;     // 上側横線
          GLCD_PutPixel(i,Yp1) ;     // 下側横線
     }
     for (i=Yp0+1 ; i < Yp1 ; i++) {
          GLCD_PutPixel(Xp0,i) ;     // 左側縦線
          GLCD_PutPixel(Xp1,i) ;     // 右側縦線
     }
}
/*******************************************************************************
*  GLCD_Line(Xp0,Yp0,Xp1,Yp1)                                                  *
*    グラフィックLCD画面の指定された位置に直線を描画させる処理                 *
*                                                                              *
*    Xp0 : 描画する直線の開始点、横アドレス(0-127)を指定する                   *
*    Yp0 : 描画する直線の開始点、縦アドレス(0-63)を指定する                    *
*    Xp1 : 描画する直線の終了点、横アドレス(0-127)を指定する                   *
*    Yp1 : 描画する直線の終了点、縦アドレス(0-63)を指定する                    *
*******************************************************************************/
void GLCD_Line(int Xp0,int Yp0,int Xp1,int Yp1)
{
     int steep ;
     int deltax, deltay, er ;
     int x, y;
     int ystep;
	
     /// 差分の大きいほうを求める
     steep = (abs(Yp1 - Yp0) > abs(Xp1 - Xp0)) ;
     /// ｘ、ｙの入れ替え
     if (steep){
          x = Xp0; Xp0 = Yp0; Yp0 = x ;
          x = Xp1; Xp1 = Yp1; Yp1 = x ;
     }
     if (Xp0 > Xp1) {
          x = Xp0; Xp0 = Xp1; Xp1 = x ;
          x = Yp0; Yp0 = Yp1; Yp1 = x ;
     }
     deltax = Xp1 - Xp0 ;		// 傾き計算
     deltay = abs(Yp1 - Yp0) ;
     er = 0 ;
     y  = Yp0 ;
     /// 傾きでステップの正負を切り替え
     if (Yp0 < Yp1) ystep = 1 ; 
     else           ystep = -1 ;
     /// 直線を点で描画
     for (x=Xp0 ; x <= Xp1 ; x++) {
          if (steep) GLCD_PutPixel(y,x) ; 
          else       GLCD_PutPixel(x,y) ;
          er += deltay ;
          if((er << 1) >= deltax) {
               y  += ystep ;
               er -= deltax ;
          }
     }
}
/*******************************************************************************
*  GLCD_PutPixel(Xp,Yp)                                                        *
*    グラフィックLCD画面の指定された位置にピクセル(ドット)を描画させる処理     *
*                                                                              *
*    Xp : 描画する位置で画面の横アドレス(0-127)を指定する                      *
*    Yp : 描画する位置で画面の縦アドレス(0-63)を指定する                       *
*******************************************************************************/
void GLCD_PutPixel(int Xp,int Yp)
{
     int col, L ;

     // ラインの選択処理
     L = 1 << (Yp % 8) ;
     // LCDに表示するアドレスの位置をセットする
     SetLocation(Xp,Yp) ;
     // LCD画面の現在表示内容に指定位置のビット(L)をON(XOR)させ、そのデータをLCDに送る
     L = ReadData() ^ L ;
     SetAddress(SetCol) ;
     WriteData(L) ;
}
/*******************************************************************************
*  SetLocation(Xp,Yp)                                                          *
*    グラフィックLCD画面の指定された位置にカーソルを移動させる処理             *
*    （64x128のピクセル値に該当するチップ選択とページ・カラムの設定を行う）    *
*                                                                              *
*    Xp : 画面の横アドレス(0-127)を指定する（チップ選択とカラム位置の計算用)   *
*    Yp : 画面の縦アドレス(0-63)を指定する（０－８ページの計算用）             *
*******************************************************************************/
void SetLocation(int Xp,int Yp)
{
     int cs ;

     // チップの選択とカラムアドレスの処理を行う
     if (Xp < 64) {
          // 左側ディスプレイのチップ(IC1)とする
          cs = 1 ;
          SetCol = Xp ;
     } else {
          // 右側ディスプレイのチップ(IC2)とする
          cs = 2 ;
          SetCol = Xp - 64 ;
     }
     // ページとラインの選択処理
     SetPg = Yp / 8 ;
     // LCDに描画するアドレスの位置を設定する
     SelectIC(cs) ;
     SetPage(SetPg) ;
     SetAddress(SetCol) ;
}
/*******************************************************************************
*  SelectIC(value)                                                             *
*    ディスプレイのチップ(IC)を選択する処理                                    *
*                                                                              *
*    value : 1=左側ディスプレイのチップ(IC1)  2=右側ディスプレイのチップ(IC2)  *
*******************************************************************************/
void SelectIC(int value)
{
     if (value==1) {
          // 左側ディスプレイのチップ(IC1)を選択する
          digitalWrite(CS1_pin, HIGH);
          digitalWrite(CS2_pin, LOW);
     } else {
          // 右側ディスプレイのチップ(IC2)を選択する
          digitalWrite(CS2_pin, HIGH);
          digitalWrite(CS1_pin, LOW);
     }
}
/*******************************************************************************
*  SetPage(value)                                                              *
*    ページの設定を行うコマンドを発行する処理                                  *
*                                                                              *
*    value : ページ０～７を選択する(縦アドレス)                                *
*******************************************************************************/
void SetPage(int value)
{
     command(0xB8|(value&0x07),LOW) ;
}
/*******************************************************************************
*  SetAddress(value)                                                           *
*    カラムアドレスの設定を行うコマンドを発行する処理                          *
*                                                                              *
*    value : ０～６３を選択する(横アドレス)                                    *
*******************************************************************************/
void SetAddress(int value)
{
     command(0x40|(value&0x3F),LOW) ;
}
/*******************************************************************************
*  ans = ReadData()                                                            *
*    現在表示されているデータの内容を読込むコマンドを発行する処理              *
*                                                                              *
*    ans : ８ビットの読込んだデータを返す                                      *
*******************************************************************************/
unsigned int ReadData()
{
     unsigned int ans , i ;

     // データピンをデジタル入力に設定
     for (i = 0; i < 8; i++) pinMode(DATA_pins[i], INPUT) ;
     // グラフィックLCDモジュールから読込みモードに設定する
     digitalWrite(RW_pin, HIGH);
     digitalWrite(RS_pin, HIGH) ;
     // データピンからデータを読込む
     digitalWrite(E_pin, HIGH) ;
     digitalWrite(E_pin, LOW) ;
     ans = 0 ;
     digitalWrite(E_pin, HIGH) ;
     for (i = 0; i < 8; i++) {
          ans = ans | (digitalRead(DATA_pins[i]) << i);
     }
     digitalWrite(E_pin, LOW) ;
     // モードを書込みに設定しデータピンをデジタル出力に戻す
     digitalWrite(RW_pin, LOW);
     for (i = 0; i < 8; i++) pinMode(DATA_pins[i], OUTPUT) ;

     return( ans ) ;
}
/*******************************************************************************
*  WriteData(value)                                                            *
*    表示するデータを書込むコマンドを発行する処理                              *
*    書込んだ後、カラムアドレスは自動的に次のアドレスへ移動します。            *
*                                                                              *
*    value : ８ビットのデータを指定する                                        *
*******************************************************************************/
void WriteData(int value)
{
     command(value,HIGH) ;
}
// グラフィックLCDモジュールコマンドを発行する処理
void command(int value,int mode)
{
     int i ;

     // RSの制御信号線をセットします
     digitalWrite(RS_pin, mode) ;
     // D0-D7のデータ線にセットします
     for (i = 0; i < 8; i++) {
          digitalWrite(DATA_pins[i], (value >> i) & 0x01) ;
     }
     // Eの制御信号線をセットします
     digitalWrite(E_pin, HIGH) ;
     digitalWrite(E_pin, LOW) ;
}
